export class RestHeaders {
    constructor(headers) {
        /** @internal header names are lower case */
        this._headers = new Map();
        /** @internal map lower case names to actual names */
        this._normalizedNames = new Map();
        if (headers instanceof RestHeaders) {
            headers.forEach((values, name) => {
                values.forEach(value => this.set(name, value));
            });
        }
        else {
            Object.keys(headers).forEach((name) => {
                const values = (Array.isArray(headers[name]) ? headers[name] : [headers[name]]);
                this.delete(name);
                values.forEach(value => this.set(name, value));
            });
        }
    }
    static from(headers) {
        if (!headers) {
            return void 0;
        }
        return new RestHeaders(headers);
    }
    /**
     * Returns a new RestHeaders instance from the given DOMString of Response RestHeaders
     */
    static fromResponseHeaderString(headersString) {
        const headers = new RestHeaders();
        headersString.split('\n').forEach(line => {
            const index = line.indexOf(':');
            if (index > 0) {
                const name = line.slice(0, index);
                const value = line.slice(index + 1).trim();
                headers.set(name, value);
            }
        });
        return headers;
    }
    /**
     * Appends a header to existing list of header values for a given header name.
     */
    append(name, value) {
        const values = this.getAll(name);
        if (values === null) {
            this.set(name, value);
        }
        else {
            values.push(value);
        }
    }
    /**
     * Deletes all header values for the given name.
     */
    delete(name) {
        const lcName = name.toLowerCase();
        this._normalizedNames.delete(lcName);
        this._headers.delete(lcName);
    }
    forEach(fn) {
        this._headers.forEach((values, lcName) => fn(values, this._normalizedNames.get(lcName), this._headers));
    }
    /**
     * Returns first header that matches given name.
     */
    get(name) {
        const values = this.getAll(name);
        if (values === null) {
            return null;
        }
        return values.length > 0 ? values[0] : null;
    }
    /**
     * Checks for existence of header by given name.
     */
    has(name) { return this._headers.has(name.toLowerCase()); }
    /**
     * Returns the names of the headers
     */
    keys() { return Array.from(this._normalizedNames.values()); }
    /**
     * Sets or overrides header value for given name.
     */
    set(name, value) {
        if (Array.isArray(value)) {
            if (value.length) {
                this._headers.set(name.toLowerCase(), [value.join(',')]);
            }
        }
        else {
            this._headers.set(name.toLowerCase(), [value]);
        }
        this.mayBeSetNormalizedName(name);
    }
    /**
     * Returns values of all headers.
     */
    values() { return Array.from(this._headers.values()); }
    /**
     * Returns string of all headers.
     */
    // TODO(vicb): returns {[name: string]: string[]}
    toJSON() {
        const serialized = {};
        if (!this._headers) {
            debugger;
        }
        this._headers.forEach((values, name) => {
            const split = [];
            values.forEach(v => split.push(...v.split(',')));
            serialized[this._normalizedNames.get(name)] = split;
        });
        return serialized;
    }
    /**
     * Returns list of header values for a given name.
     */
    getAll(name) {
        return this.has(name) ? this._headers.get(name.toLowerCase()) : null;
    }
    mayBeSetNormalizedName(name) {
        const lcName = name.toLowerCase();
        if (!this._normalizedNames.has(lcName)) {
            this._normalizedNames.set(lcName, name);
        }
    }
}
//# sourceMappingURL=rest-headers.js.map